import maya.cmds as cmds
import math

def bake(_objs, _frame_step_count):
    start = cmds.playbackOptions(q=True, min=True)
    end  = cmds.playbackOptions(q=True, max=True)

    print("The timeline starts at frame " + str(start) + " and ends at " + str(end) + ".")
    print("The frame step count is " + str(_frame_step_count) + ".")

    nextFrameIdx = int(start);
    copyBuffers = []
    for obj in _objs:
        copyBuffers.append([])

    while nextFrameIdx <= end:
        cmds.currentTime(nextFrameIdx)
        cmds.refresh()
        print("Current frame is", nextFrameIdx)
        for i,obj in enumerate(_objs):
            cpy = cmds.duplicate(obj, name="baked_" + format(nextFrameIdx, '04d'))
            copyBuffers[i].append(cpy[0])
        nextFrameIdx += _frame_step_count

    for buf in copyBuffers:
        #select the blend targets first and then the initial frame so that the blendshape will be applied to it
        cmds.select(buf[1:], replace=True)
        cmds.select(buf[0], add=True)
        bs = cmds.blendShape(ib=True, tc=False)
        
        #delete all blend state meshes
        cmds.select(buf[1:], replace=True)
        cmds.delete()

        # cmds.select(bs[0], replace=True)
        cmds.setKeyframe(bs, attribute=buf[-1], time=start, value=0)
        cmds.setKeyframe(bs, attribute=buf[-1], time=end, value=1)
        cmds.selectKey(bs)
        cmds.keyTangent( itt='linear', ott='linear' )

def showUI():
    selection = cmds.ls(sl=True)

    if len(selection) < 1:
        cmds.confirmDialog(title='Error', message='No objects selected', button=['Ok'], defaultButton='Ok')
        return
    else:
        for obj in selection:
            for node in cmds.listHistory(obj):
                print("NODE ON OBJ", node, cmds.nodeType(node))

    window = cmds.window( title="Long Name", iconName='Short Name', widthHeight=(200, 100) )

    # As we add contents to the window, align them vertically
    cmds.columnLayout( adjustableColumn=True )

    # A button that does nothing
    cmds.text("Frame Sample Distance")

    sampleDistance = 10
    def numberChange(num):
        nonlocal sampleDistance
        sampleDistance = num

    sampleCountField = cmds.intField( ann='Frame Sample Distance', changeCommand=numberChange, value=sampleDistance)

    def foo(*args):
        bake(selection, sampleDistance)
        cmds.deleteUI(window, window = True)

    # Close button with a command to delete the UI
    cmds.button( label='Ok', command=foo)
    cmds.button( label='Close', command=('cmds.deleteUI(\"' + window + '\", window=True)') )

    # Set its parent to the Maya window (denoted by '..')
    cmds.setParent( '..' )

    # Show the window that we created (window)
    cmds.showWindow( window )

showUI()
